
from typing import Optional, List, Tuple

from cadquery import Workplane
from paramak import Shape


class SweepMixedShape(Shape):
    """Sweeps a 2D shape created from points connected with straight, spline
    or circle connections along a defined spline path to create a 3D CadQuery
    solid. Note, some variation in cross-section of the solid may occur.

    Args:
        path_points: A list of XY, YZ or XZ coordinates connected by spline
            connections which define the path along which the 2D shape is
            swept.
        workplane: Workplane in which the 2D shape to be swept is defined.
            Defaults to "XY".
        path_workplane: Workplane in which the spline path is defined. Defaults
            to "XZ".
        force_cross_section (bool, optional): If True, cross-section of solid
            is forced to be shape defined by points in workplane at each
            path_point. Defaults to False.
    """

    def __init__(
        self,
        path_points: List[Tuple[float, float]],
        workplane: Optional[str] = "XY",
        path_workplane: Optional[str] = "XZ",
        force_cross_section: Optional[bool] = False,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.792, 0.698, 0.839),
        **kwargs
    ):

        super().__init__(
            workplane=workplane,
            color=color,
            **kwargs
        )

        self.path_points = path_points
        self.path_workplane = path_workplane
        self.force_cross_section = force_cross_section

    @property
    def path_points(self):
        return self._path_points

    @path_points.setter
    def path_points(self, value):
        self._path_points = value

    @property
    def path_workplane(self):
        return self._path_workplane

    @path_workplane.setter
    def path_workplane(self, value):
        if value[0] != self.workplane[0]:
            raise ValueError(
                "workplane and path_workplane must start with the same letter"
            )
        elif value == self.workplane:
            raise ValueError(
                "workplane and path_workplane must be different"
            )
        else:
            self._path_workplane = value

    def create_solid(self):
        """Creates a swept 3D solid from a 2D shape with mixed connections.

        Returns:
            A CadQuery solid: A 3D solid volume
        """

        solid = super().create_solid()
        path = Workplane(self.path_workplane).spline(self.path_points)

        wire = solid.close()

        self.wire = wire

        solid = wire.sweep(path, multisection=True)

        solid = self.rotate_solid(solid)
        solid = self.perform_boolean_operations(solid)
        self.solid = solid
        return solid




import os
import unittest
import warnings
from pathlib import Path

import paramak


class TestBlanketFP(unittest.TestCase):

    def setUp(self):
        self.plasma = paramak.Plasma(
            major_radius=450,
            minor_radius=150,
            triangularity=0.55,
            elongation=2
        )

        self.test_shape = paramak.BlanketFP(
            thickness=150,
            start_angle=-90,
            stop_angle=240,
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a BlanketFP are correct."""

        assert self.test_shape.plasma is None
        assert self.test_shape.minor_radius == 150
        assert self.test_shape.major_radius == 450
        assert self.test_shape.triangularity == 0.55
        assert self.test_shape.elongation == 2
        assert self.test_shape.vertical_displacement == 0
        assert self.test_shape.offset_from_plasma == 0
        assert self.test_shape.num_points == 50

    def test_creation_plasma(self):
        """Checks that a cadquery solid can be created by passing a plasma to
        the BlanketFP parametric component."""

        self.test_shape.plasma = self.plasma
        self.test_shape.offset_from_plasma = 30

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_faces(self):
        """creates a blanket using the BlanketFP parametric component and checks
        that a solid with the correct number of faces is created"""

        self.test_shape.plasma = self.plasma
        self.test_shape.offset_from_plasma = 30

        assert len(self.test_shape.areas) == 4
        assert len(set([round(i) for i in self.test_shape.areas])) == 4

        self.test_shape.rotation_angle = 180
        assert len(self.test_shape.areas) == 6
        assert len(set([round(i) for i in self.test_shape.areas])) == 5

    def test_creation_noplasma(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when no plasma is passed."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_creation_variable_thickness_from_tuple(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when a tuple of thicknesses is passed as an
        argument."""

        self.test_shape.thickness = (100, 200)

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_creation_variable_thickness_from_2_lists(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when a list of angles and a list of thicknesses
        are passed as an argument."""

        self.test_shape.thickness = [(-90, 240), [10, 30]]

        assert self.test_shape.solid is not None

    def test_creation_variable_thickness_function(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when a thickness function is passed as an
        argument."""

        def thickness(theta):
            return 10 + 0.1 * theta

        self.test_shape.thickness = thickness

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_creation_variable_offset_from_tuple(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when a tuple of offsets is passed as an
        argument."""

        self.test_shape.offset_from_plasma = (0, 10)

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_creation_variable_offset_from_2_lists(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when a list of offsets and a list of angles are
        passed as an argument."""

        self.test_shape.start_angle = 90
        self.test_shape.stop_angle = 270
        self.test_shape.offset_from_plasma = [[270, 100, 90], [0, 5, 10]]

        assert self.test_shape.solid is not None

    def test_creation_variable_offset_error(self):
        """Checks that an error is raised when two lists with different
        lengths are passed in offset_from_plasma as an argument."""

        def test_different_lengths():
            self.test_shape.start_angle = 90
            self.test_shape.stop_angle = 270
            self.test_shape.offset_from_plasma = [
                [270, 100, 90], [0, 5, 10, 15]]
            self.test_shape.solid

        self.assertRaises(ValueError, test_different_lengths)

    def test_creation_variable_offset_function(self):
        """Checks that a cadquery solid can be created using the BlanketFP
        parametric component when an offset function is passed."""

        def offset(theta):
            return 10 + 0.1 * theta

        self.test_shape.offset_from_plasma = offset

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_full_cov_stp_export(self):
        """Creates a blanket using the BlanketFP parametric component with full
        coverage and checks that an stp file can be exported using the export_stp
        method."""

        self.test_shape.rotation_angle = 180
        self.test_shape.start_angle = 0
        self.test_shape.stop_angle = 360

        self.test_shape.export_stp("test_blanket_full_cov.stp")
        assert Path("test_blanket_full_cov.stp").exists()
        os.system("rm test_blanket_full_cov.stp")

    def test_full_cov_full_rotation(self):
        """Creates a blanket using the BlanketFP parametric component with full
        coverage and full rotation and checks that an stp file can be exported using
        the export_stp method."""

        self.test_shape.rotation_angle = 360
        self.test_shape.start_angle = 0
        self.test_shape.stop_angle = 360

        self.test_shape.export_stp("test_blanket_full_cov_full_rot.stp")
        assert Path("test_blanket_full_cov_full_rot.stp").exists()
        os.system("rm test_blanket_full_cov_full_rot.stp")

    def test_overlapping(self):
        """Creates an overlapping geometry and checks that a warning is raised.
        """

        test_shape = paramak.BlanketFP(
            major_radius=100,
            minor_radius=100,
            triangularity=0.5,
            elongation=2,
            thickness=200,
            stop_angle=360,
            start_angle=0,
        )

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            for a in w:
                print(a.message)
            assert test_shape.solid is not None
            assert "BlanketFP: Some points with negative R" in str(
                w[0].message)




import unittest

import paramak

import pytest


class TestPortCutterRectangular(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.PortCutterRectangular(
            width=20, height=40, distance=300
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a PortCutterRectangular are correct."""

        assert self.test_shape.center_point == (0, 0)
        assert self.test_shape.workplane == "ZY"
        assert self.test_shape.rotation_axis == "Z"
        assert self.test_shape.extrusion_start_offset == 1
        assert self.test_shape.fillet_radius is None
        assert self.test_shape.name == "rectangular_port_cutter"

    def test_creation(self):
        """Creates a rectangular port cutter using the PortCutterRectangular parametric
        component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_relative_volume(self):
        """Creates PortCutterRectangular shapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()

        self.test_shape.extrusion_start_offset = 20
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]

        assert self.test_shape.volume() == pytest.approx(test_volume * 4)

    def test_absolute_volume(self):
        """Creates a PortCutterRectangular shape and checks that its volume is correct."""

        assert self.test_shape.volume() == pytest.approx(20 * 40 * 300)

        self.test_shape.extrusion_start_offset = 20
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        self.test_shape.width = 20
        self.test_shape.height = 20

        assert self.test_shape.volume() == pytest.approx(20 * 20 * 300 * 4)

    def test_workplane(self):
        """Creates PortCutterRectangular shapes in different workplanes and checks that
        the geometries are correct."""

        cutting_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 50), (500, 50), (500, 0)],
            workplane="YZ",
        )
        self.test_shape.cut = cutting_shape

        assert self.test_shape.volume() == pytest.approx(20 * 40 * 300 * 0.5)

        self.test_shape.workplane = "XZ"
        cutting_shape.workplane = "YZ"

        assert self.test_shape.volume() == pytest.approx(20 * 40 * 300 * 0.5)

    def test_filleting(self):
        """Creates a PortCutterRectangular shape with filleted edges and checks
        that its volume is correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.fillet_radius = 5

        assert self.test_shape.volume() < test_volume
        self.test_shape.workplane = "ZX"
        assert self.test_shape.volume() < test_volume
        self.test_shape.workplane = "YX"
        assert self.test_shape.volume() < test_volume




import math
import unittest

import paramak
import pytest


class TestPoloidalFieldCoilCase(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.PoloidalFieldCoilCase(
            casing_thickness=5,
            coil_height=50,
            coil_width=50,
            center_point=(1000, 500)
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a PoloidalFieldCoilCase are correct."""

        assert self.test_shape.rotation_angle == 360

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the
        InnerTfCoilsFlat component are calculated correctly from the parameters
        given."""

        assert self.test_shape.processed_points == [
            (1025.0, 525.0, 'straight'),
            (1025.0, 475.0, 'straight'),
            (975.0, 475.0, 'straight'),
            (975.0, 525.0, 'straight'),
            (1025.0, 525.0, 'straight'),
            (1030.0, 530.0, 'straight'),
            (1030.0, 470.0, 'straight'),
            (970.0, 470.0, 'straight'),
            (970.0, 530.0, 'straight'),
            (1030.0, 530.0, 'straight'),
            (1025.0, 525.0, 'straight')
        ]

    def test_points_calculation(self):
        """Checks that the points used to construct the InnerTfCoilsFlat
        component are calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (1025.0, 525.0),
            (1025.0, 475.0),
            (975.0, 475.0),
            (975.0, 525.0),
            (1025.0, 525.0),
            (1030.0, 530.0),
            (1030.0, 470.0),
            (970.0, 470.0),
            (970.0, 530.0),
            (1030.0, 530.0),
        ]

    def test_creation(self):
        """Creates a pf coil case using the PoloidalFieldCoilCase parametric
        component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_absolute_volume(self):
        """Creates a pf coil case using the PoloidalFieldCoilCase parametric
        component and checks that its volume is correct."""

        assert self.test_shape.volume() == pytest.approx(
            (math.pi * 2 * 1000) * ((60 * 5 * 2) + (50 * 5 * 2)))

    def test_absolute_areas(self):
        """Creates a pf coil case using the PoloidalFieldCoilCase parametric
        component and checks that the areas of its faces are correct."""

        assert len(self.test_shape.areas) == 8
        assert len(set([round(i) for i in self.test_shape.areas])) == 6
        assert self.test_shape.areas.count(
            pytest.approx(50 * math.pi * 2 * 1000)) == 2
        assert self.test_shape.areas.count(
            pytest.approx(60 * math.pi * 2 * 1000)) == 2
        assert self.test_shape.areas.count(
            pytest.approx(50 * math.pi * 2 * 1025)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(50 * math.pi * 2 * 975)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(60 * math.pi * 2 * 1030)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(60 * math.pi * 2 * 970)) == 1




from typing import Optional, Tuple

from paramak import RotateMixedShape


class RotateStraightShape(RotateMixedShape):
    """Rotates a 3d CadQuery solid from points connected with straight
    connections.

    Args:
        rotation_angle: The rotation angle to use when revolving the solid
            (degrees).
    """

    def __init__(
        self,
        rotation_angle: Optional[float] = 360.,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.89, 0.101, 0.109),
        **kwargs
    ):

        super().__init__(
            rotation_angle=rotation_angle,
            color=color,
            connection_type="straight",
            **kwargs
        )




import unittest

import paramak


class TestDivertorITER(unittest.TestCase):

    def test_creation(self):
        """Creates an ITER-type divertor using the ITERtypeDivertor parametric
        component and checks that a cadquery solid is created"""

        test_shape = paramak.ITERtypeDivertor()
        assert test_shape.solid is not None

    def test_stp_export(self):
        """Creates an ITER-type divertor using the ITERtypeDivertor parametric
        component and checks that a stp file of the shape can be exported using
        the export_stp method"""

        test_shape = paramak.ITERtypeDivertor()
        test_shape.export_stp("tests/ITER_div.stp")

    def test_faces(self):
        """Creates an ITER-type divertor using the ITERtypeDivertor parametric
        component and checks that a solid with the correct number of faces is
        created"""

        test_shape = paramak.ITERtypeDivertor()
        assert len(test_shape.areas) == 12
        assert len(set(test_shape.areas)) == 12

        test_shape.rotation_angle = 180
        assert len(test_shape.areas) == 14
        assert len(set(test_shape.areas)) == 13




from typing import Optional, Tuple

from paramak import RotateMixedShape


class RotateSplineShape(RotateMixedShape):
    """Rotates a 3d CadQuery solid from points connected with splines.

    Args:
        rotation_angle: The rotation_angle to use when revolving the solid.
            Defaults to 360.0.
    """

    def __init__(
        self,
        rotation_angle: Optional[float] = 360.,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.415, 0.239, 0.603),
        **kwargs
    ):

        super().__init__(
            rotation_angle=rotation_angle,
            color=color,
            connection_type="spline",
            **kwargs
        )




import math
from typing import Optional
import numpy as np
from paramak import ExtrudeStraightShape


class InnerTfCoilsFlat(ExtrudeStraightShape):
    """A tf coil volume with straight inner and outer profiles and
    constant gaps between each coil. Note: the inner / outer surface is not
    equal distance to the center point everywhere as the corners are further
    than the straight sections.

    Args:
        height: height of tf coils.
        inner_radius: Distance between center point and the inner surface of
            the tf coils.
        outer_radius: Distance between center point and the outer surface of
            the tf coils.
        number_of_coils: number of tf coils.
        gap_size: gap between adjacent tf coils.
        radius_type: Controls the part of the inner surface used when
            defining the inner_radius and outer_radius. Can be set to either
            'corner' or 'straight'.
        azimuth_start_angle: defaults to 0.0.
        workplane:defaults to "XY".
        rotation_axis: Defaults to "Z".
    """

    def __init__(
        self,
        height: float,
        inner_radius: float,
        outer_radius: float,
        number_of_coils: int,
        gap_size: float,
        radius_type: Optional[str] = 'corner',
        azimuth_start_angle: Optional[float] = 0.0,
        workplane: Optional[str] = "XY",
        rotation_axis: Optional[str] = "Z",
        **kwargs
    ) -> None:

        super().__init__(
            distance=height,
            workplane=workplane,
            rotation_axis=rotation_axis,
            **kwargs
        )

        self.azimuth_start_angle = azimuth_start_angle
        self.height = height
        self.inner_radius = inner_radius
        self.outer_radius = outer_radius
        self.radius_type = radius_type
        self.number_of_coils = number_of_coils
        self.gap_size = gap_size
        self.distance = height

    @property
    def azimuth_start_angle(self):
        return self._azimuth_start_angle

    @azimuth_start_angle.setter
    def azimuth_start_angle(self, value):
        self._azimuth_start_angle = value

    @property
    def radius_type(self):
        return self._radius_type

    @radius_type.setter
    def radius_type(self, value):
        if value not in ['corner', 'straight']:
            msg = (
                f'radius_type must be either "corner" or "straight". Not {value}')
            raise ValueError(msg)
        self._radius_type = value

    @property
    def azimuth_placement_angle(self):
        self.find_azimuth_placement_angle()
        return self._azimuth_placement_angle

    @azimuth_placement_angle.setter
    def azimuth_placement_angle(self, value):
        self._azimuth_placement_angle = value

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, height):
        self._height = height

    @property
    def distance(self):
        return self.height

    @distance.setter
    def distance(self, value):
        self._distance = value

    @property
    def inner_radius(self):
        return self._inner_radius

    @inner_radius.setter
    def inner_radius(self, inner_radius):
        self._inner_radius = inner_radius

    @property
    def outer_radius(self):
        return self._outer_radius

    @outer_radius.setter
    def outer_radius(self, outer_radius):
        self._outer_radius = outer_radius

    @property
    def number_of_coils(self):
        return self._number_of_coils

    @number_of_coils.setter
    def number_of_coils(self, number_of_coils):
        self._number_of_coils = number_of_coils

    @property
    def gap_size(self):
        return self._gap_size

    @gap_size.setter
    def gap_size(self, gap_size):
        self._gap_size = gap_size

    def find_points(self):
        """Finds the points that describe the 2D profile of the tf coil shape"""

        #       /   p4
        #      /    /¦
        #     /    / ¦
        #    /    /  ¦
        #   /    /   ¦
        #     p1/    ¦
        #      ¦     ¦
        # x    ¦     ¦
        #      ¦     ¦
        #     p2\    ¦
        #   \    \   ¦
        #    \    \  ¦
        #     \    \ ¦
        #      \    p3

        if self.radius_type == 'corner':
            distance_to_inner_corner = self.inner_radius
            distance_to_rear_corner = self.outer_radius
        # this section calculates a new distance to the corners now that we
        # know the user provided the distance to the straight
        if self.radius_type == 'straight':
            angle = 360 / (self.number_of_coils * 2)
            distance_to_inner_corner = self.inner_radius / \
                math.cos(math.radians(angle))
            distance_to_rear_corner = self.outer_radius / \
                math.cos(math.radians(angle))

        if self.gap_size * self.number_of_coils > 2 * math.pi * distance_to_inner_corner:
            msg = (
                'Gap_size is too large. The gap_size * number of coils must '
                'be less than the circumference of the circle made by '
                'the inner_radius')
            raise ValueError(msg)

        if distance_to_inner_corner != 0.:
            theta_inner = (
                (2 * math.pi * distance_to_inner_corner) - (self.gap_size * self.number_of_coils)
            ) / (distance_to_inner_corner * self.number_of_coils)
            omega_inner = math.asin(
                self.gap_size / (2 * distance_to_inner_corner))

            # inner points
            point_1 = (
                (distance_to_inner_corner * math.cos(-omega_inner)),
                (-distance_to_inner_corner * math.sin(-omega_inner)),
            )
            point_2 = (
                (
                    distance_to_inner_corner * math.cos(theta_inner) * math.cos(-omega_inner)
                    + distance_to_inner_corner * math.sin(theta_inner) * math.sin(-omega_inner)
                ),
                (
                    -distance_to_inner_corner * math.cos(theta_inner) * math.sin(-omega_inner)
                    + distance_to_inner_corner * math.sin(theta_inner) * math.cos(-omega_inner)
                ),
            )
            points = [
                (point_1[0], point_1[1]),
                (point_2[0], point_2[1])
            ]

        else:

            points = [(0, 0)]

        # print(point_1)
        # print(point_2)

        theta_outer = (
            (2 * math.pi * distance_to_rear_corner) - (self.gap_size * self.number_of_coils)
        ) / (distance_to_rear_corner * self.number_of_coils)
        omega_outer = math.asin(self.gap_size / (2 * distance_to_rear_corner))

        # outer points
        point_4 = (
            (distance_to_rear_corner * math.cos(-omega_outer)),
            (-distance_to_rear_corner * math.sin(-omega_outer)),
        )
        point_6 = (
            (
                distance_to_rear_corner * math.cos(theta_outer) * math.cos(-omega_outer)
                + distance_to_rear_corner * math.sin(theta_outer) * math.sin(-omega_outer)
            ),
            (
                -distance_to_rear_corner * math.cos(theta_outer) * math.sin(-omega_outer)
                + distance_to_rear_corner * math.sin(theta_outer) * math.cos(-omega_outer)
            ),
        )
        points.append((point_6[0], point_6[1]))
        points.append((point_4[0], point_4[1]))

        self.points = points

    def find_azimuth_placement_angle(self):
        """Calculates the azimuth placement angles based on the number of tf
        coils"""

        angles = list(
            np.linspace(
                0 + self.azimuth_start_angle,
                360 + self.azimuth_start_angle,
                self.number_of_coils,
                endpoint=False))

        self.azimuth_placement_angle = angles



import math
import unittest

import paramak

import pytest


class TestPortCutterCircular(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.PortCutterCircular(
            distance=300, radius=20
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a PortCutterCircular are correct."""

        assert self.test_shape.center_point == (0, 0)
        assert self.test_shape.workplane == "ZY"
        assert self.test_shape.rotation_axis == "Z"
        assert self.test_shape.extrusion_start_offset == 1
        assert self.test_shape.name == "circular_port_cutter"

    def test_creation(self):
        """Creates a circular port cutter using the PortCutterCircular parametric
        component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_relative_volume(self):
        """Creates PortCutterCircular shapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()

        self.test_shape.extrusion_start_offset = 20
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]

        assert self.test_shape.volume() == pytest.approx(test_volume * 4)

    def test_absolute_volume(self):
        """Creates a PortCutterCircular shape and checks that its volume is correct."""

        assert self.test_shape.volume() == pytest.approx(math.pi * (20**2) * 300)

        self.test_shape.extrusion_start_offset = 20
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        self.test_shape.radius = 10

        assert self.test_shape.volume() == pytest.approx(
            math.pi * (10**2) * 300 * 4)




from typing import Optional
from paramak import RotateStraightShape


class CuttingWedge(RotateStraightShape):
    """Creates a wedge from height, radius and rotation angle arguments than
    can be useful for cutting sector models.

    Args:
        height: the vertical (z axis) height of the coil (cm).
        radius: the horizontal (x axis) width of the coil (cm).
        rotation_angle: Defaults to 180.0.
    """

    def __init__(
        self,
        height: float,
        radius: float,
        rotation_angle: Optional[float] = 180.0,
        **kwargs
    ) -> None:

        super().__init__(
            rotation_angle=rotation_angle,
            **kwargs
        )

        self.height = height
        self.radius = radius

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, value):
        self._height = value

    @property
    def radius(self):
        return self._radius

    @radius.setter
    def radius(self, value):
        self._radius = value

    def find_points(self):

        points = [
            (0, self.height / 2),
            (self.radius, self.height / 2),
            (self.radius, -self.height / 2),
            (0, -self.height / 2)
        ]

        self.points = points



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="paramak",
    version="0.6.1",
    author="The Paramak Development Team",
    author_email="mail@jshimwell.com",
    description="Create 3D fusion reactor CAD models based on input parameters",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fusion-energy/paramak",
    packages=setuptools.find_packages(),
    classifiers=[
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=[
        "plotly>=5.1.0",
        "scipy>=1.7.0",
        "sympy>=1.8",
        "numpy>=1.21.1",
        "matplotlib>=3.4.2",
        "plasmaboundaries>=0.1.8",
        "jupyter-client<7",
        "jupyter-cadquery>=2.2.0",
    ])




import unittest

import paramak


class TestBlanketCutterParallels(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.BlanketCutterParallels(
            thickness=50,
            gap_size=200
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a BlanketCutterParallel are correct."""

        assert self.test_shape.azimuth_placement_angle == [
            0., 36., 72., 108., 144., 180., 216., 252., 288., 324.]
        assert self.test_shape.height == 2000
        assert self.test_shape.width == 2000

    def test_creation(self):
        """Creates solid using the BlanketCutterParallels parametric component
        and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_BlanketCutterParallels_distance_volume_impact(self):
        """Creates solid using the BlanketCutterParallels parametric component
        with a larger thickness and checks that the volume changes accordingly
        ."""

        test_volume = self.test_shape.volume()
        self.test_shape.thickness = 100
        assert test_volume < self.test_shape.volume()

    def test_cut_modification(self):
        """Creates a BlanketCutterParallels parametric component and with another
        shape cut out and checks that a solid can be produced."""

        cut_shape = paramak.ExtrudeCircleShape(1, 1, points=[(0, 0)])
        self.test_shape.cut = cut_shape
        assert self.test_shape.solid is not None

    def test_distance_is_modified(self):
        test_shape = paramak.BlanketCutterParallels(
            thickness=50,
            gap_size=50,
        )

        for thickness, gap_size in zip([20, 30, 40], [10, 20, 30]):
            test_shape.thickness = thickness
            test_shape.gap_size = gap_size
            assert test_shape.distance == test_shape.gap_size / 2 + test_shape.thickness

    def test_main_cutting_shape_is_modified(self):
        test_shape = paramak.BlanketCutterParallels(
            thickness=50,
            gap_size=50,
        )

        for gap_size, angles in zip([10, 20, 30], [0, 1, 3]):
            test_shape.gap_size = gap_size
            test_shape.azimuth_placement_angle = angles
            assert test_shape.main_cutting_shape.distance == test_shape.gap_size / 2.0
            assert test_shape.main_cutting_shape.azimuth_placement_angle == test_shape.azimuth_placement_angle




import unittest

import paramak


class TestPoloidalSegments(unittest.TestCase):

    def test_solid_count_with_incorrect_input(self):
        """Checks the segmenter does not take a float as an input."""

        def create_shape():
            test_shape_to_segment = paramak.PoloidalFieldCoil(
                height=100,
                width=100,
                center_point=(500, 500)
            )

            paramak.PoloidalSegments(
                shape_to_segment=test_shape_to_segment,
                center_point=(500, 500),
                number_of_segments=22.5,
            )

        self.assertRaises(
            TypeError, create_shape)

    def test_solid_count_with_incorrect_inputs2(self):
        """Checks the segmenter does not take a negative int as an input."""

        def create_shape():
            test_shape_to_segment = paramak.PoloidalFieldCoil(
                height=100,
                width=100,
                center_point=(500, 500)
            )

            paramak.PoloidalSegments(
                shape_to_segment=test_shape_to_segment,
                center_point=(500, 500),
                number_of_segments=-5,
            )

        self.assertRaises(
            ValueError, create_shape)

    def test_solid_count(self):
        """Creates a rotated hollow ring and segments it into poloidal
        sections."""

        pf_coil = paramak.PoloidalFieldCoil(
            height=100,
            width=100,
            center_point=(500, 500)
        )

        test_shape_to_segment = paramak.PoloidalFieldCoilCaseFC(
            pf_coil=pf_coil,
            casing_thickness=100
        )

        test_shape = paramak.PoloidalSegments(
            shape_to_segment=test_shape_to_segment,
            center_point=(500, 500),
            number_of_segments=22,
        )

        assert test_shape.solid is not None
        assert len(test_shape.solid.Solids()) == 22

    def test_solid_count2(self):
        """Creates a rotated ring and segments it into poloidal sections."""

        test_shape_to_segment = paramak.PoloidalFieldCoil(
            height=100,
            width=100,
            center_point=(500, 500)
        )

        test_shape = paramak.PoloidalSegments(
            shape_to_segment=test_shape_to_segment,
            center_point=(500, 500),
            number_of_segments=22,
        )

        assert test_shape.solid is not None
        assert len(test_shape.solid.Solids()) == 22

    def test_without_shape_to_segment(self):
        """Checks a solid can be created if no shape is given
        """
        test_shape = paramak.PoloidalSegments(
            shape_to_segment=None,
            center_point=(500, 500),
            number_of_segments=22,
        )

        assert test_shape.solid is not None




import os
import unittest
from pathlib import Path

import pytest
from cadquery import Plane
from paramak import ExtrudeStraightShape


class TestExtrudeStraightShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = ExtrudeStraightShape(
            points=[(10, 10), (10, 30), (30, 30), (30, 10)], distance=30
        )

    def test_workplane_of_type_cadquery_plane(self):
        """Tests that a Cadquery.Plane is accepted as a workplane entry and
        makes a shape with the same volume as the default 'XY' workplane"""

        normal_vec = (1, 1, 1)
        workplane = Plane(origin=(0, 0, 0), xDir=(-1, 1, 0), normal=normal_vec)
        # in future releases of CQ, origin and xDir will be optional

        test_shape_2 = ExtrudeStraightShape(
            workplane=workplane,
            rotation_axis='X',
            points=[(10, 10), (10, 30), (30, 30), (30, 10)], distance=30
        )

        assert isinstance(test_shape_2.workplane, Plane)
        assert pytest.approx(test_shape_2.volume, self.test_shape.volume)

    def test_default_parameters(self):
        """Checks that the default parameters of an ExtrudeStraightShape are
        correct."""

        assert self.test_shape.rotation_angle == 360
        assert self.test_shape.extrude_both

    def test_absolute_shape_volume(self):
        """Creates an ExtrudeStraightShape and checks that the volume is
        correct."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() == pytest.approx(20 * 20 * 30)

    def test_absolute_shape_areas(self):
        """Creates an ExtrudeStraightShape and checks that the volume is
        correct."""

        assert self.test_shape.area == pytest.approx(
            (20 * 20 * 2) + (20 * 30 * 4)
        )
        assert len(self.test_shape.areas) == 6
        assert self.test_shape.areas.count(pytest.approx(20 * 20)) == 2
        assert self.test_shape.areas.count(pytest.approx(20 * 30)) == 4

    def test_relative_shape_volume(self):
        """Creates two ExtrudeStraightShapes and checks that their relative
        volumes are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        self.test_shape.rotation_axis = "Y"
        assert self.test_shape.volume() == pytest.approx(test_volume * 4)

    def test_cut_volume(self):
        """Creates an ExtrudeStraightShape with another ExtrudeStraightShape cut
        out and checks that the volume is correct."""

        shape_with_cut = ExtrudeStraightShape(
            points=[(0, 0), (0, 40), (40, 40), (40, 0)], distance=40,
            cut=self.test_shape
        )

        assert shape_with_cut.volume() == pytest.approx(
            (40 * 40 * 40) - (20 * 20 * 30)
        )

    def test_union_volume(self):
        """Creates a union of two ExtrudeStraightShapes and checks that the
        volume is correct."""

        unioned_shape = ExtrudeStraightShape(
            points=[(0, 10), (0, 30), (20, 30), (20, 10)], distance=30,
            union=self.test_shape
        )
        assert unioned_shape.volume() == pytest.approx(30 * 20 * 30)

    def test_intersect_volume(self):
        """Creates an ExtrudeStraightShape with another ExtrudeStraightShape
        intersected and checks that the volume is correct."""

        intersected_shape = ExtrudeStraightShape(
            points=[(0, 10), (0, 30), (20, 30), (20, 10)], distance=30,
            intersect=self.test_shape
        )
        assert intersected_shape.volume() == pytest.approx(10 * 20 * 30)

    def test_rotation_angle(self):
        """Creates an ExtrudeStraightShape with a rotation_angle < 360 and
        checks that the correct cut is performed and the volume is correct."""

        self.test_shape.azimuth_placement_angle = [45, 135, 225, 315]
        self.rotation_axis = "Y"
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.5, rel=0.01)

    def test_extrude_both(self):
        """Creates an ExtrudeStraightShape with extrude_both = True and False
        and checks that the volumes are correct."""

        test_volume_extrude_both = self.test_shape.volume()
        self.test_shape.extrude_both = False
        assert self.test_shape.volume() == pytest.approx(
            test_volume_extrude_both)

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks that the
        outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert Path("test_solid.stp").stat().st_size == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

    def test_incorrect_points_input(self):
        """Checks that an error is raised when the points are input with the
        connection"""

        def incorrect_points_definition():
            self.test_shape.points = [
                (10, 10, 'straight'),
                (10, 30, 'straight'),
                (30, 30, 'straight'),
                (30, 10, 'straight')
            ]

        self.assertRaises(
            ValueError,
            incorrect_points_definition
        )

    def test_export_html_with_different_workplanes(self):
        """Checks that all the workplanes produce an html file when using the
        export_html method and that the axis have the correct labels"""

        os.system("rm *.html")
        for workplane in ["XY", "YZ", "XZ", "YX", "ZY", "ZX"]:
            self.test_shape.workplane = workplane
            fig = self.test_shape.export_html(workplane + ".html")
            assert Path(workplane + ".html").exists() is True
            assert fig.layout.xaxis.title['text'] == workplane[0]
            assert fig.layout.yaxis.title['text'] == workplane[1]


if __name__ == "__main__":
    unittest.main()




import unittest

import paramak
import pytest


class TestBlanketCutterStar(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.BlanketCutterStar(distance=100)

    def test_default_parameters(self):
        """Checks that the default parameters of a BlanketCutterStar are correct."""

        assert self.test_shape.azimuth_placement_angle == [
            0., 36., 72., 108., 144., 180., 216., 252., 288., 324.]
        assert self.test_shape.height == 2000
        assert self.test_shape.width == 2000
        assert self.test_shape.name == "blanket_cutter_star"

    def test_points_calculation(self):
        """Checks that the points used to construct the BlanketCutterStar component
        are calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (0, -1000),
            (2000, -1000),
            (2000, 1000),
            (0, 1000),
        ]

    def test_processed_points_calculation(self):
        """Checks that the points used to construct the BlanketCutterStar component
        are calculated correctly from the parameters given."""

        assert self.test_shape.processed_points == [
            (0, -1000, "straight"),
            (2000, -1000, "straight"),
            (2000, 1000, "straight"),
            (0, 1000, "straight"),
            (0, -1000, "straight")
        ]

    def test_creation(self):
        """Creates a solid using the BlanketCutterStar parametric component
        and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_distance_volume_impact(self):
        """Creates solid using the BlanketCutterStar parametric component
        with different distances and checks that the volume changes accordingly
        ."""

        test_volume = self.test_shape.volume()
        self.test_shape.distance = 50
        # not quite two times as large as there is overlap in the center
        assert test_volume == pytest.approx(
            self.test_shape.volume() * 2, rel=0.1)




from paramak import Plasma


class PlasmaFromPoints(Plasma):
    """Creates a double null tokamak plasma shape that is controlled by 3
    coordinates.

    Args:
        outer_equatorial_x_point (float): the x value of the outer equatorial
            of the plasma (cm).
        inner_equatorial_x_point (float): the x value of the inner equatorial
            of the plasma (cm).
        high_point (tuple of 2 floats): the (x,z) coordinate values of the top
            of the plasma (cm).
    """

    def __init__(
        self,
        outer_equatorial_x_point,
        inner_equatorial_x_point,
        high_point,
        **kwargs
    ):

        minor_radius = (outer_equatorial_x_point -
                        inner_equatorial_x_point) / 2.0
        major_radius = inner_equatorial_x_point + minor_radius
        elongation = high_point[1] / minor_radius
        triangularity = (major_radius - high_point[0]) / minor_radius

        super().__init__(
            elongation=elongation,
            major_radius=major_radius,
            minor_radius=minor_radius,
            triangularity=triangularity,
            **kwargs
        )

        self.outer_equatorial_x_point = outer_equatorial_x_point
        self.inner_equatorial_x_point = inner_equatorial_x_point
        self.high_point = high_point

    @property
    def outer_equatorial_x_point(self):
        return self._outer_equatorial_x_point

    @outer_equatorial_x_point.setter
    def outer_equatorial_x_point(self, value):
        self._outer_equatorial_x_point = value

    @property
    def inner_equatorial_x_point(self):
        return self._inner_equatorial_x_point

    @inner_equatorial_x_point.setter
    def inner_equatorial_x_point(self, value):
        self._inner_equatorial_x_point = value

    @property
    def high_point(self):
        return self._high_point

    @high_point.setter
    def high_point(self, value):
        self._high_point = value




import os
import time
import unittest
import warnings
from pathlib import Path

import paramak


class TestBallReactor(unittest.TestCase):
    """Tests functionality of the BallReactor class"""

    def setUp(self):
        self.test_reactor = paramak.BallReactor(
            inner_bore_radial_thickness=50,
            inboard_tf_leg_radial_thickness=200,
            center_column_shield_radial_thickness=50,
            divertor_radial_thickness=100,
            inner_plasma_gap_radial_thickness=150,
            plasma_radial_thickness=100,
            outer_plasma_gap_radial_thickness=50,
            plasma_gap_vertical_thickness=50,
            firstwall_radial_thickness=50,
            blanket_radial_thickness=100,
            blanket_rear_wall_radial_thickness=10,
            elongation=2,
            triangularity=0.55,
            number_of_tf_coils=16,
            rotation_angle=180,
        )

    def test_input_variable_names(self):
        """tests that the number of inputs variables is correct"""

        assert len(self.test_reactor.input_variables.keys()) == 28
        assert len(self.test_reactor.input_variable_names) == 28

    def test_creation_with_narrow_divertor(self):
        """Creates a BallReactor with a narrow divertor and checks that the correct
        number of components are created."""

        self.test_reactor.divertor_radial_thickness = 50

        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 7

    def test_creation_with_wide_divertor(self):
        """Creates a BallReactor with a wide divertor and checks that the correct
        number of components are created."""

        self.test_reactor.divertor_radial_thickness = 172.5

        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 7

    def test_svg_creation(self):
        """Creates a BallReactor and checks that an svg image of the reactor can be
        exported using the export_svg method."""

        os.system("rm test_ballreactor_image.svg")
        self.test_reactor.export_svg("filename.svg")
        assert Path("filename.svg").exists() is True
        os.system("rm filename.svg")

    def test_with_pf_coils(self):
        """Checks that a BallReactor with optional pf coils can be created and that
        the correct number of components are created."""

        self.test_reactor.pf_coil_radial_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_position = [200, 200, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [200, 100, -100, -200]
        self.test_reactor.pf_coil_case_thickness = 10

        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 9

    def test_rotation_angle_error(self):
        """Checks that an error is raised when an angle is over 360."""

        def invalid_rotation_angle():
            self.test_reactor.rotation_angle = 361
        self.assertRaises(ValueError, invalid_rotation_angle)

    def test_pf_coil_thicknesses_error(self):
        """Checks that an error is raised when invalid pf_coil_radial_thicknesses and
        pf_coil_vertical_thicknesses are specified."""

        def invalid_pf_coil_radial_thicknesses():
            self.test_reactor.pf_coil_radial_thicknesses = 2
        self.assertRaises(ValueError, invalid_pf_coil_radial_thicknesses)

        def invalid_pf_coil_vertical_thicknesses():
            self.test_reactor.pf_coil_vertical_thicknesses = 2
        self.assertRaises(ValueError, invalid_pf_coil_vertical_thicknesses)

    def test_with_pf_and_tf_coils(self):
        """Checks that a BallReactor with optional pf and tf coils can be created and
        that the correct number of components are created."""

        self.test_reactor.pf_coil_radial_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_position = [200, 200, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [200, 100, -100, -200]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thickness = 10
        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 50

        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 10

    def test_rotation_angle_warning(self):
        """Creates a BallReactor with rotation_angle = 360 and checks that the correct
        warning message is printed."""

        def warning_trigger():
            self.test_reactor.rotation_angle = 360

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            warning_trigger()
            assert len(w) == 1
            assert issubclass(w[-1].category, UserWarning)
            assert "360 degree rotation may result in a Standard_ConstructionError or AttributeError" in str(
                w[-1].message)

    def test_ball_reactor_hash_value(self):
        """Creates a ball reactor and checks that all shapes in the reactor are
        created when .shapes_and_components is first called. Checks that when
        .shapes_and_components is called again with no changes to the reactor,
        the shapes in the reactor are not reconstructed and the previously
        constructed shapes are returned. Checks that when
        .shapes_and_components is called again with changes to the reactor, the
        shapes in the reactor are reconstructed and these new shapes are
        returned. Checks that the reactor_hash_value is only updated when the
        reactor is reconstructed."""

        self.test_reactor.pf_coil_radial_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_position = [200, 200, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [200, 100, -100, -200]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thickness = 10
        self.test_reactor.outboard_tf_coil_radial_thickness = 100
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 50

        assert self.test_reactor.reactor_hash_value is None
        for key in [
            "_plasma",
            "_inboard_tf_coils",
            "_center_column_shield",
            "_divertor",
            "_firstwall",
            "_blanket",
            "_blanket_rear_wall",
            "_pf_coil",
            "_pf_coils_casing",
            "_tf_coil"
        ]:
            assert key not in self.test_reactor.__dict__.keys()
        assert self.test_reactor.shapes_and_components is not None

        for key in [
            "_plasma",
            "_inboard_tf_coils",
            "_center_column_shield",
            "_divertor",
            "_firstwall",
            "_blanket",
            "_blanket_rear_wall",
            "_pf_coil",
            "_pf_coils_casing",
            "_tf_coil"
        ]:
            assert key in self.test_reactor.__dict__.keys()
        assert len(self.test_reactor.shapes_and_components) == 10
        assert self.test_reactor.reactor_hash_value is not None
        initial_hash_value = self.test_reactor.reactor_hash_value
        self.test_reactor.rotation_angle = 270
        assert self.test_reactor.reactor_hash_value == initial_hash_value
        assert self.test_reactor.shapes_and_components is not None
        assert self.test_reactor.reactor_hash_value != initial_hash_value

    def test_hash_value_time_saving(self):
        """Checks that use of conditional reactor reconstruction via the hash value
        gives the expected time saving."""

        self.test_reactor.pf_coil_radial_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_position = [200, 200, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [200, 100, -100, -200]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thickness = 10
        self.test_reactor.outboard_tf_coil_radial_thickness = 100
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 50

        start_time = time.time()
        self.test_reactor.shapes_and_components
        stop_time = time.time()
        initial_construction_time = stop_time - start_time

        start_time = time.time()
        self.test_reactor.shapes_and_components
        stop_time = time.time()
        reconstruction_time = stop_time - start_time

        assert reconstruction_time < initial_construction_time
        # assert reconstruction_time < initial_construction_time * 0.01

    def test_divertor_position_error(self):
        """checks an invalid divertor position raises the correct
        ValueError."""

        def invalid_position():
            self.test_reactor.divertor_position = "coucou"

        self.assertRaises(ValueError, invalid_position)

    def test_divertor_upper_lower(self):
        """Checks that BallReactors with coils with lower and upper divertors
        can be created."""
        self.test_reactor.pf_coil_radial_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_position = [200, 200, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [200, 100, -100, -200]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thickness = 10
        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 50

        self.test_reactor.divertor_position = "lower"
        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 10

        self.test_reactor.divertor_position = "upper"
        assert self.test_reactor.solid is not None
        assert len(self.test_reactor.shapes_and_components) == 10

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs folders exist."""

        os.system("rm reactor_solids.stp")
        os.system("rm reactor_wires.stp")

        self.test_reactor.export_stp(
            filename='reactor_solid.stp',
            mode='solid'
        )

        self.test_reactor.export_stp(
            filename='reactor_wires.stp',
            mode='wire'
        )

        assert Path("reactor_solid.stp").is_file()
        assert Path("reactor_wires.stp").is_file()




import sys
import os

import nbformat
from nbconvert.preprocessors import ExecutePreprocessor
from nbconvert.preprocessors.execute import CellExecutionError


def notebook_run(path):
    """
    Execute a notebook via nbconvert and collect output.
    :returns (parsed nb object, execution errors)
    """
    kernel_name = 'python%d' % sys.version_info[0]
    this_file_directory = os.path.dirname(__file__)
    errors = []

    with open(path) as file:
        note_book = nbformat.read(file, as_version=4)
        note_book.metadata.get('kernelspec', {})['name'] = kernel_name
        ep = ExecutePreprocessor(
            kernel_name=kernel_name,
            timeout=800)

        try:
            ep.preprocess(
                note_book, {
                    'metadata': {
                        'path': this_file_directory}})

        except CellExecutionError as e:
            if "SKIP" in e.traceback:
                print(str(e.traceback).split("\n")[-2])
            else:
                raise e

    return errors




import math
import os
import unittest
from pathlib import Path

import pytest
from paramak import RotateStraightShape


class TestRotateStraightShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20), (20, 0)]
        )

    def test_largest_dimension(self):
        """Checks that the largest_dimension is correct."""

        assert self.test_shape.largest_dimension == 20

    def test_default_parameters(self):
        """Checks that the default parameters of a RotateStraightShape are correct."""

        assert self.test_shape.rotation_angle == 360
        assert self.test_shape.azimuth_placement_angle == 0

    def test_rotation_angle_getting_setting(self):
        """Checks that the rotation_angle of a RotateStraightShape can be changed."""

        assert self.test_shape.rotation_angle == 360
        self.test_shape.rotation_angle = 180
        assert self.test_shape.rotation_angle == 180

    def test_absolute_shape_volume(self):
        """Creates a RotateStraightShape and checks that its volume is correct."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() == pytest.approx(math.pi * (20**2) * 20)

    def test_relative_shape_volume(self):
        """Creates two RotateStraightShapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert test_volume == pytest.approx(self.test_shape.volume() * 2)

    def test_union_volume_addition(self):
        """Fuses two RotateStraightShapes and checks that their fused volume is
        correct."""

        inner_box = RotateStraightShape(
            points=[(100, 100), (100, 200), (200, 200), (200, 100)],
        )

        outer_box = RotateStraightShape(
            points=[(200, 100), (200, 200), (500, 200), (500, 100)],
        )

        outer_box_and_inner_box = RotateStraightShape(
            points=[(200, 100), (200, 200), (500, 200), (500, 100)],
            union=inner_box,
        )

        assert inner_box.volume() + outer_box.volume() == pytest.approx(
            outer_box_and_inner_box.volume(), rel=0.01
        )

    def test_absolute_shape_areas(self):
        """Creates RotateStraightShapes and checks that the areas of each face
        are correct."""

        assert self.test_shape.area == pytest.approx(
            (math.pi * (20**2) * 2) + (math.pi * (20 * 2) * 20))
        assert len(self.test_shape.areas) == 3
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (20**2))) == 2
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (20 * 2) * 20)) == 1

        self.test_shape.rotation_angle = 180
        assert self.test_shape.area == pytest.approx(
            ((math.pi * (20**2) / 2) * 2) + (20 * 40) + (math.pi * (20 * 2) * 20 / 2))
        assert len(self.test_shape.areas) == 4
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (20**2) / 2)) == 2
        assert self.test_shape.areas.count(pytest.approx(20 * 40)) == 1
        assert self.test_shape.areas.count(
            pytest.approx((math.pi * (20 * 2) * 20) / 2)) == 1

        test_shape = RotateStraightShape(
            points=[(50, 0), (50, 50), (70, 50), (70, 0)],
        )

        assert test_shape.area == pytest.approx(
            (((math.pi * (70**2)) - (math.pi * (50**2))) * 2) + (math.pi * (50 * 2) * 50) + (math.pi * (70 * 2) * 50))
        assert len(test_shape.areas) == 4
        assert test_shape.areas.count(pytest.approx(
            (math.pi * (70**2)) - (math.pi * (50**2)))) == 2
        assert test_shape.areas.count(
            pytest.approx(math.pi * (50 * 2) * 50)) == 1
        assert test_shape.areas.count(
            pytest.approx(math.pi * (70 * 2) * 50)) == 1

        test_shape.rotation_angle = 180
        assert test_shape.area == pytest.approx((20 * 50 * 2) + ((((math.pi * (70**2)) / 2) - (
            (math.pi * (50**2)) / 2)) * 2) + ((math.pi * (50 * 2) * 50) / 2) + ((math.pi * (70 * 2) * 50) / 2))
        assert len(test_shape.areas) == 6
        assert test_shape.areas.count(pytest.approx(20 * 50)) == 2
        assert test_shape.areas.count(pytest.approx(
            ((math.pi * (70**2)) / 2) - ((math.pi * (50**2)) / 2))) == 2
        assert test_shape.areas.count(
            pytest.approx(math.pi * (50 * 2) * 50 / 2)) == 1
        assert test_shape.areas.count(
            pytest.approx(math.pi * (70 * 2) * 50 / 2)) == 1

    def test_export_stp_extension(self):
        """Creates a RotateStraightShape and checks that a stp file of the
        shape can be exported with the correct suffix using the export_stp
        method."""

        os.system("rm filename.stp filename.step")
        self.test_shape.export_stp("filename.stp")
        self.test_shape.export_stp("filename.step")
        assert Path("filename.stp").exists() is True
        assert Path("filename.step").exists() is True
        os.system("rm filename.stp filename.step")

    def test_export_stp_extension_in_cm(self):
        """Creates a RotateStraightShape and checks that a stp file of the
        shape can be exported with the correct suffix using the export_stp
        method."""

        os.system("rm filename.stp filename.step")
        self.test_shape.export_stp("filename.stp", units='cm')
        self.test_shape.export_stp("filename.step", units='cm')
        assert Path("filename.stp").exists() is True
        assert Path("filename.step").exists() is True
        os.system("rm filename.stp filename.step")
        self.test_shape.export_stp("filename_cm.stp", units='cm')
        assert Path("filename_cm.stp").exists() is True
        os.system("rm filename_cm.stp")

    def test_export_stl(self):
        """Creates a RotateStraightShape and checks that a stl file of the
        shape can be exported with the correct suffix using the export_stl
        method."""

        os.system("rm filename.stl")
        self.test_shape.export_stl("filename.stl")
        assert Path("filename.stl").exists() is True
        os.system("rm filename.stl")

    def test_export_svg(self):
        """Creates a RotateStraightShape and checks that a svg file of the
        shape can be exported with the correct suffix using the export_svg
        method."""

        os.system("rm filename.svg")
        self.test_shape.export_svg("filename.svg")
        assert Path("filename.svg").exists() is True
        os.system("rm filename.svg")
        self.test_shape.export_svg("filename")
        assert Path("filename.svg").exists() is True
        os.system("rm filename.svg")

    def test_export_svg_options(self):
        """Creates a RotateStraightShape and checks that a svg file of the
        shape can be exported with the various different export options"""

        os.system("rm *.svg")
        self.test_shape.export_svg("width.svg", width=900)
        assert Path("width.svg").exists() is True
        self.test_shape.export_svg("height.svg", height=900)
        assert Path("height.svg").exists() is True
        self.test_shape.export_svg("marginLeft.svg", marginLeft=110)
        assert Path("marginLeft.svg").exists() is True
        self.test_shape.export_svg("marginTop.svg", marginTop=110)
        assert Path("marginTop.svg").exists() is True
        self.test_shape.export_svg("showAxes.svg", showAxes=True)
        assert Path("showAxes.svg").exists() is True
        self.test_shape.export_svg(
            "projectionDir.svg", projectionDir=(-1, -1, -1))
        assert Path("projectionDir.svg").exists() is True
        self.test_shape.export_svg("strokeColor.svg", strokeColor=(42, 42, 42))
        assert Path("strokeColor.svg").exists() is True
        self.test_shape.export_svg("hiddenColor.svg", hiddenColor=(42, 42, 42))
        assert Path("hiddenColor.svg").exists() is True
        self.test_shape.export_svg("showHidden.svg", showHidden=False)
        assert Path("showHidden.svg").exists() is True
        self.test_shape.export_svg("strokeWidth1.svg", strokeWidth=None)
        assert Path("strokeWidth1.svg").exists() is True
        self.test_shape.export_svg("strokeWidth2.svg", strokeWidth=10)
        assert Path("strokeWidth2.svg").exists() is True

    def test_cut_volume(self):
        """Creates a RotateStraightShape with another RotateStraightShape
        cut out and checks that the volume is correct."""

        shape_with_cut = RotateStraightShape(
            points=[(0, -5), (0, 25), (25, 25), (25, -5)],
            cut=self.test_shape
        )

        assert shape_with_cut.volume() == pytest.approx(
            (math.pi * (25**2) * 30) - (math.pi * (20**2) * 20)
        )

    def test_multiple_cut_volume(self):
        """Creates a RotateStraightShape with multiple RotateStraightShapes
        cut out and checks that the volume is correct."""

        main_shape = RotateStraightShape(
            points=[(0, 0), (0, 200), (200, 200), (200, 0)],
        )

        shape_to_cut_1 = RotateStraightShape(
            points=[(20, 0), (20, 200), (40, 200), (40, 0)],
        )

        shape_to_cut_2 = RotateStraightShape(
            points=[(120, 0), (120, 200), (140, 200), (140, 0)],
        )

        main_shape_with_cuts = RotateStraightShape(
            points=[(0, 0), (0, 200), (200, 200), (200, 0)],
            cut=[shape_to_cut_1, shape_to_cut_2]
        )

        assert main_shape_with_cuts.volume() == pytest.approx(
            (math.pi * (200**2) * 200) -
            ((math.pi * (40**2) * 200) - (math.pi * (20**2) * 200)) -
            ((math.pi * (140**2) * 200) - (math.pi * (120**2) * 200))
        )

    def test_hash_value(self):
        """Creates a RotateStraightShape and checks that a cadquery solid with
        a unique has value is created when .solid is called. Checks that the same
        solid is returned when .solid is called again after no changes have been
        made to the shape. Checks that a new solid with a new unique has is
        constructed when .solid is called after changes to the shape have been
        made. Checks that the hash_value of the shape is not updated until a new
        solid has been constructed."""

        assert self.test_shape.hash_value is None
        assert self.test_shape.solid is not None
        assert self.test_shape.hash_value is not None
        initial_hash_value = self.test_shape.hash_value
        assert self.test_shape.solid is not None
        assert initial_hash_value == self.test_shape.hash_value
        self.test_shape.rotation_angle = 180
        assert initial_hash_value == self.test_shape.hash_value
        assert self.test_shape.solid is not None
        assert initial_hash_value != self.test_shape.hash_value

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert Path("test_solid.stp").stat().st_size == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

    def test_export_stp_with_incorrect_args(self):
        """Checks errors are raised when incorrect arguments are used
        """

        def export_mode_incorrect():
            self.test_shape.export_stp(
                'test_solid.stp',
                mode='coucou'
            )

        self.assertRaises(ValueError, export_mode_incorrect)

    def test_graveyard_filename(self):
        """Checks the name of the stp file for the Graveyard is correct
        """
        output_filename = self.test_shape.export_graveyard()
        assert 'graveyard.stp' == output_filename

        output_filename = self.test_shape.export_graveyard(filename='test.stp')
        assert 'test.stp' == output_filename

    def test_incorrect_points_input(self):
        """Checks that an error is raised when the points are input with the
        connection"""

        def incorrect_points_definition():
            self.test_shape.points = [
                (10, 10, 'straight'),
                (10, 30, 'straight'),
                (30, 30, 'straight'),
                (30, 10, 'straight')
            ]

        self.assertRaises(
            ValueError,
            incorrect_points_definition
        )


if __name__ == "__main__":
    unittest.main()




import os
import unittest
from pathlib import Path

import paramak
import pytest


class TestSubmersionTokamak(unittest.TestCase):
    """Tests functionality of the SubmersionTokamak class"""

    def setUp(self):
        self.test_reactor = paramak.SubmersionTokamak(
            inner_bore_radial_thickness=10,
            inboard_tf_leg_radial_thickness=30,
            center_column_shield_radial_thickness=60,
            divertor_radial_thickness=50,
            inner_plasma_gap_radial_thickness=30,
            plasma_radial_thickness=300,
            outer_plasma_gap_radial_thickness=30,
            firstwall_radial_thickness=30,
            blanket_rear_wall_radial_thickness=30,
            number_of_tf_coils=16,
            outboard_tf_coil_radial_thickness=10,
            outboard_tf_coil_poloidal_thickness=10,
            rear_blanket_to_tf_gap=10,
            support_radial_thickness=20,
            inboard_blanket_radial_thickness=20,
            outboard_blanket_radial_thickness=20,
            elongation=2.3,
            triangularity=0.45,
            rotation_angle=359,
        )

    def test_input_variable_names(self):
        """tests that the number of inputs variables is correct"""

        assert len(self.test_reactor.input_variables.keys()) == 29
        assert len(self.test_reactor.input_variable_names) == 29

    def test_svg_creation(self):
        """Creates a SubmersionTokamak and checks that an svg file of the
        reactor can be exported using the export_svg method."""

        os.system("rm test_image.svg")
        self.test_reactor.export_svg("test_image.svg")

        assert Path("test_image.svg").exists() is True
        os.system("rm test_image.svg")

    def test_minimal_creation(self):
        """Creates a SubmersionTokamak and checks that the correct number of
        components are created."""

        assert len(self.test_reactor.shapes_and_components) == 9

    def test_with_tf_coils_creation(self):
        """Creates a SubmersionTokamak with tf coils and checks that the correct
        number of components are created."""

        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_radial_position = [100]
        self.test_reactor.pf_coil_vertical_position = [100]
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 70
        self.test_reactor.number_of_tf_coils = 4

        assert len(self.test_reactor.shapes_and_components) == 9

    def test_with_tf_and_pf_coils_creation(self):
        """Creates a SubmersionTokamak with tf and pf coils and checks that the
        correct number of components are created."""

        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 70
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50, 50, 50, 50]
        self.test_reactor.pf_coil_radial_thicknesses = [40, 40, 40, 40, 40]
        self.test_reactor.pf_coil_radial_position = [100, 100, 200, 200, 100]
        self.test_reactor.pf_coil_vertical_position = [100, -100, 200, -200, 0]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thicknesses = 10
        self.test_reactor.number_of_tf_coils = 4

        assert len(self.test_reactor.shapes_and_components) == 11

    def test_minimal_stp_creation(self):
        """Creates a SubmersionTokamak and checks that stp files of all
        components can be exported using the export_stp method."""

        os.system("rm -r minimal_SubmersionTokamak")
        output_filenames = [
            "minimal_SubmersionTokamak/inboard_tf_coils.stp",
            "minimal_SubmersionTokamak/center_column_shield.stp",
            "minimal_SubmersionTokamak/plasma.stp",
            "minimal_SubmersionTokamak/divertor.stp",
            "minimal_SubmersionTokamak/outboard_firstwall.stp",
            "minimal_SubmersionTokamak/supports.stp",
            "minimal_SubmersionTokamak/blanket.stp",
            "minimal_SubmersionTokamak/outboard_rear_blanket_wall.stp",
            "another.stp",
        ]
        self.test_reactor.export_stp(filename=output_filenames)

        for output_filename in output_filenames:
            assert Path(output_filename).exists() is True
        os.system("rm -r minimal_SubmersionTokamak")

    def test_with_pf_coils_stp_creation(self):
        """Creates a SubmersionTokamak with pf coils and checks that stp files
        of all components can be exported using the export_stp method."""

        os.system("rm -r pf_SubmersionTokamak")

        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50]
        self.test_reactor.pf_coil_radial_thicknesses = [40, 40]
        self.test_reactor.pf_coil_radial_position = [100, 100]
        self.test_reactor.pf_coil_vertical_position = [100, -100]

        output_filenames = [
            "pf_SubmersionTokamak/inboard_tf_coils.stp",
            "pf_SubmersionTokamak/center_column_shield.stp",
            "pf_SubmersionTokamak/plasma.stp",
            "pf_SubmersionTokamak/divertor.stp",
            "pf_SubmersionTokamak/outboard_firstwall.stp",
            "pf_SubmersionTokamak/supports.stp",
            "pf_SubmersionTokamak/blanket.stp",
            "pf_SubmersionTokamak/outboard_rear_blanket_wall.stp",
            "tf_pf_SubmersionTokamak/another_shape.stp",
            "tf_pf_SubmersionTokamak/another_shape2.stp",
            "tf_pf_SubmersionTokamak/another_shape3.stp",
        ]
        self.test_reactor.export_stp(filename=output_filenames)

        for output_filename in output_filenames:
            assert Path(output_filename).exists() is True
        os.system("rm -r pf_SubmersionTokamak")

    def test_with_tf_and_pf_coils_stp_creation(self):
        """Creates a SubmersionTokamak with tf and pf coils and checks that
        stp files of all components can be exported using the export_stp method."""

        os.system("rm -r tf_pf_SubmersionTokamak")

        self.test_reactor.outboard_tf_coil_radial_thickness = 50
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 70
        self.test_reactor.pf_coil_vertical_thicknesses = [50, 50]
        self.test_reactor.pf_coil_radial_thicknesses = [40, 40]
        self.test_reactor.pf_coil_radial_position = [100, 100]
        self.test_reactor.pf_coil_vertical_position = [100, -100]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thicknesses = 10
        self.test_reactor.number_of_tf_coils = 4

        output_filenames = [
            "tf_pf_SubmersionTokamak/inboard_tf_coils.stp",
            "tf_pf_SubmersionTokamak/center_column_shield.stp",
            "tf_pf_SubmersionTokamak/plasma.stp",
            "tf_pf_SubmersionTokamak/divertor.stp",
            "tf_pf_SubmersionTokamak/outboard_firstwall.stp",
            "tf_pf_SubmersionTokamak/supports.stp",
            "tf_pf_SubmersionTokamak/blanket.stp",
            "tf_pf_SubmersionTokamak/outboard_rear_blanket_wall.stp",
            "tf_pf_SubmersionTokamak/pf_coil_cases.stp",
            "tf_pf_SubmersionTokamak/another1.stp",
            "tf_pf_SubmersionTokamak/another2.stp"
        ]
        self.test_reactor.export_stp(filename=output_filenames)

        for output_filename in output_filenames:
            assert Path(output_filename).exists() is True
        os.system("rm -r tf_pf_SubmersionTokamak")

    def test_rotation_angle_warning(self):
        """Creates a SubmersionTokamak with rotation_angle = 360 and checks that the
        correct warning message is printed."""

        def warning_trigger():
            try:
                self.test_reactor.rotation_angle = 360
                self.test_reactor._rotation_angle_check()
            except BaseException:
                pass
        msg = "360 degree rotation may result in a " + \
            "Standard_ConstructionError or AttributeError"
        with pytest.warns(UserWarning, match=msg):
            warning_trigger()

    def test_submersion_reactor_hash_value(self):
        """Creates a submersion reactor and checks that all shapes in the reactor are created
        when .shapes_and_components is first called. Checks that when .shapes_and_components
        is called again with no changes to the reactor, the shapes in the reactor are
        reconstructed and the previously constructed shapes are returned. Checks that when
        .shapes_and_components is called again with no changes to the reactor, the shapes in
        the reactor are reconstructed and these new shapes are returned. Checks that the
        reactor_hash_value is only updated when the reactor is reconstructed."""

        self.test_reactor.pf_coil_radial_thicknesses = [30, 30, 30, 30]
        self.test_reactor.pf_coil_vertical_thicknesses = [30, 30, 30, 30]
        self.test_reactor.rear_blanket_to_tf_gap = 50
        self.test_reactor.pf_coil_case_thicknesses = 10
        self.test_reactor.outboard_tf_coil_radial_thickness = 30
        self.test_reactor.outboard_tf_coil_poloidal_thickness = 30
        self.test_reactor.pf_coil_radial_position = [100, 100, 200, 200]
        self.test_reactor.pf_coil_vertical_position = [100, -100, 200, -200]
        self.test_reactor.number_of_tf_coils = 16

        assert self.test_reactor.reactor_hash_value is None
        for key in [
            "_inboard_tf_coils",
            "_center_column_shield",
            "_plasma",
            "_inboard_firstwall",
            "_inboard_blanket",
            "_firstwall",
            "_divertor",
            "_blanket",
            "_supports",
            "_outboard_rear_blanket_wall_upper",
            "_outboard_rear_blanket_wall_lower",
            "_outboard_rear_blanket_wall",
            "_tf_coil",
            "_pf_coil",
            "_pf_coils_casing"
        ]:
            assert key not in self.test_reactor.__dict__

        assert self.test_reactor.shapes_and_components is not None
        for key in [
            "_inboard_tf_coils",
            "_center_column_shield",
            "_plasma",
            "_inboard_firstwall",
            "_inboard_blanket",
            "_firstwall",
            "_divertor",
            "_blanket",
            "_supports",
            "_outboard_rear_blanket_wall_upper",
            "_outboard_rear_blanket_wall_lower",
            "_outboard_rear_blanket_wall",
            "_tf_coil",
            "_pf_coil",
            "_pf_coils_casing"
        ]:
            assert key in self.test_reactor.__dict__.keys()

        assert len(self.test_reactor.shapes_and_components) == 11
        assert self.test_reactor.reactor_hash_value is not None
        initial_hash_value = self.test_reactor.reactor_hash_value
        self.test_reactor.rotation_angle = 270
        assert self.test_reactor.reactor_hash_value == initial_hash_value
        assert self.test_reactor.shapes_and_components is not None
        assert self.test_reactor.reactor_hash_value != initial_hash_value

    def test_error_divertor_pos(self):
        """Checks an invalid divertor and support
        position raises the correct ValueError."""

        def invalid_divertor_position():
            self.test_reactor.divertor_position = "coucou"

        self.assertRaises(ValueError, invalid_divertor_position)

        def invalid_support_position():
            self.test_reactor.support_position = "coucou"

        self.assertRaises(ValueError, invalid_support_position)

    def test_divertors_supports(self):
        """Checks that SubmersionTokamaks with lower and upper supports
        and divertors can be created."""

        self.test_reactor.divertor_position = "lower"
        self.test_reactor.support_position = "lower"
        assert self.test_reactor.solid is not None

        self.test_reactor.divertor_position = "lower"
        self.test_reactor.support_position = "upper"
        assert self.test_reactor.solid is not None

        self.test_reactor.divertor_position = "upper"
        self.test_reactor.support_position = "lower"
        assert self.test_reactor.solid is not None

        self.test_reactor.divertor_position = "upper"
        self.test_reactor.support_position = "upper"
        assert self.test_reactor.solid is not None




from paramak import Plasma
from plasmaboundaries import get_separatrix_coordinates


class PlasmaBoundaries(Plasma):
    """Creates a double null tokamak plasma shape that is controlled
    by 5 shaping parameters using the plasmaboundaries package to calculate
    points. For more details see:
    http://github.com/RemDelaporteMathurin/plasma-boundaries

    Args:
        A (float, optional): plasma parameter see plasmaboundaries doc.
            Defaults to 0.05.
        elongation (float, optional): the elongation of the plasma.
            Defaults to 2.0.
        major_radius (float, optional): the major radius of the plasma
            (cm). Defaults to 450.0.
        minor_radius (float, optional): the minor radius of the plasma
            (cm). Defaults to 150.0.
        triangularity (float, optional): the triangularity of the plasma.
            Defaults to 0.55.
        vertical_displacement (float, optional): the vertical_displacement
            of the plasma (cm). Defaults to 0.0.
        configuration (str, optional): plasma configuration
            ("non-null", "single-null", "double-null").
            Defaults to "non-null".
        x_point_shift (float, optional): Shift parameters for locating the
            X points in [0, 1]. Defaults to 0.1.
    """

    def __init__(
        self,
        A=0.05,
        elongation=2.0,
        major_radius=450.0,
        minor_radius=150.0,
        triangularity=0.55,
        vertical_displacement=0.0,
        configuration="non-null",
        x_point_shift=0.1,
        **kwargs
    ):

        super().__init__(
            elongation=elongation,
            major_radius=major_radius,
            minor_radius=minor_radius,
            triangularity=triangularity,
            vertical_displacement=vertical_displacement,
            configuration=configuration,
            x_point_shift=x_point_shift,
            **kwargs
        )

        # properties needed for plasma shapes
        self.A = A

    def find_points(self):
        """Finds the XZ points that describe the 2D profile of the plasma."""
        aspect_ratio = self.minor_radius / self.major_radius
        params = {
            "A": self.A,
            "aspect_ratio": aspect_ratio,
            "elongation": self.elongation,
            "triangularity": self.triangularity,
        }
        points = get_separatrix_coordinates(
            params, self.configuration)
        # add vertical displacement
        points[:, 1] += self.vertical_displacement
        # rescale to cm
        points[:] *= self.major_radius

        # remove unnecessary points
        # if non-null these are the y bounds
        lower_point_y = self.low_point[1]
        upper_point_y = self.high_point[1]
        # else use x points
        if self.configuration in ["single-null", "double-null"]:
            lower_point_y = self.lower_x_point[1]
            if self.configuration == "double-null":
                upper_point_y = self.upper_x_point[1]

        points = points[
            (points[:, 1] >= lower_point_y) & (points[:, 1] <= upper_point_y)]
        self.points = points.tolist()[:-1]




import unittest

import paramak
import pytest


class TestBlanketConstantThicknessArcH(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.BlanketConstantThicknessArcH(
            inner_lower_point=(300, -200),
            inner_mid_point=(500, 0),
            inner_upper_point=(300, 200),
            thickness=20,
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a BlanketConstantThicknessArcH are correct."""

        assert self.test_shape.rotation_angle == 360

    def test_points_calculation(self):
        """Checks that the points used to construct the BlanketConstantThicknessArcH component
        are calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (300, 200, 'circle'),
            (500, 0, 'circle'),
            (300, -200, 'straight'),
            (320, -200, 'circle'),
            (520, 0, 'circle'),
            (320, 200, 'straight')
        ]

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the
        BlanketConstantThicknessArcH component are calculated correctly from
        the parameters given."""

        assert self.test_shape.processed_points == [
            (300, 200, 'circle'),
            (500, 0, 'circle'),
            (300, -200, 'straight'),
            (320, -200, 'circle'),
            (520, 0, 'circle'),
            (320, 200, 'straight'),
            (300, 200, 'circle')
        ]

    def test_component_creation(self):
        """Creates a blanket using the BlanketConstantThicknessArcH parametric
        component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_relative_shape_volume(self):
        """Creates two blankets using the BlanketConstantThicknessArcH parametric component
        and checks that their relative volumes are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert test_volume == pytest.approx(self.test_shape.volume() * 2)

    def test_shape_face_areas(self):
        """Creates a blanket using the BlanketConstantThicknessArcH parametric component and
        checks that the face areas are expected."""

        assert len(self.test_shape.areas) == 4
        assert len(set([round(i) for i in self.test_shape.areas])) == 3

        self.test_shape.rotation_angle = 180
        assert len(self.test_shape.areas) == 6
        assert len(set([round(i) for i in self.test_shape.areas])) == 4




import math

import cadquery as cq
from paramak import RotateStraightShape
from paramak.utils import (coefficients_of_line_from_points, intersect_solid,
                           rotate)


class PoloidalSegments(RotateStraightShape):
    """Creates a ring of wedges from a central point. When provided with a
    shape_to_segment the shape will be segmented by the wedges. This is useful
    for segmenting geometry into equal poloidal angles. Intended to segment the
    firstwall geometry for using in neutron wall loading simulations.

    Args:
        center_point (tuple of floats): the center of the segmentation wedges
            (x,z) values (cm).
        shape_to_segment (paramak.Shape, optional): the Shape to segment, if
            None then the segmenting solids will be returned. Defaults to None.
        number_of_segments (int, optional): the number of equal angles
            segments in 360 degrees. Defaults to 10.
        max_distance_from_center (float): the maximum distance from the center
            point outwards (cm). Defaults to 1000.0.
        name (str, optional): defaults to "poloidal_segmenter".
    """

    def __init__(
        self,
        center_point,
        shape_to_segment=None,
        number_of_segments=10,
        max_distance_from_center=1000.0,
        name="poloidal_segmenter",
        **kwargs
    ):

        super().__init__(
            name=name,
            **kwargs
        )

        self.center_point = center_point
        self.shape_to_segment = shape_to_segment
        self.number_of_segments = number_of_segments
        self.max_distance_from_center = max_distance_from_center

    @property
    def number_of_segments(self):
        return self._number_of_segments

    @number_of_segments.setter
    def number_of_segments(self, value):
        if isinstance(value, int) is False:
            raise TypeError(
                "PoloidalSegmenter.number_of_segments must be an int.")
        if value < 1:
            raise ValueError(
                "PoloidalSegmenter.number_of_segments must be a minimum of 1.")
        self._number_of_segments = value

    @property
    def shape_to_segment(self):
        return self._shape_to_segment

    @shape_to_segment.setter
    def shape_to_segment(self, value):
        self._shape_to_segment = value

    @property
    def center_point(self):
        return self._center_point

    @center_point.setter
    def center_point(self, center_point):
        self._center_point = center_point

    @property
    def max_distance_from_center(self):
        return self._max_distance_from_center

    @max_distance_from_center.setter
    def max_distance_from_center(self, value):
        self._max_distance_from_center = value

    def find_points(self):
        """Finds the XZ points joined by straight connections that describe
        the 2D profile of the poloidal segmentation shape."""

        angle_per_segment = 360. / self.number_of_segments

        points = []

        current_angle = 0

        outer_point = (
            self.center_point[0] +
            self.max_distance_from_center,
            self.center_point[1])
        for i in range(self.number_of_segments):

            points.append(self.center_point)

            outer_point_1 = rotate(
                self.center_point,
                outer_point,
                math.radians(current_angle)
            )

            outer_point_2 = rotate(
                self.center_point,
                outer_point,
                math.radians(current_angle + angle_per_segment)
            )

            # if the point goes beyond the zero line then set to zero
            for new_point in [outer_point_1, outer_point_2]:
                if new_point[0] < 0:
                    m, c = coefficients_of_line_from_points(
                        new_point, self.center_point)
                    points.append((0, c))
                else:
                    points.append(new_point)

            current_angle = current_angle + angle_per_segment

        self.points = points

    def create_solid(self):
        """Creates a 3d solid using points with straight edges. Individual
        solids in the compound can be accessed using .Solids()[i] where i is an
        int.

           Returns:
              A CadQuery solid: A 3D solid volume
        """

        iter_points = iter(self.points)
        triangle_wedges = []
        wires = []
        for p1, p2, p3 in zip(iter_points, iter_points, iter_points):

            solid = (
                cq.Workplane(self.workplane)
                .polyline([p1[:2], p2[:2], p3[:2]])
            )

            wire = solid.close()

            wires.append(wire)

            solid = wire.revolve(self.rotation_angle)

            triangle_wedges.append(solid)

        self.wire = wires

        if self.shape_to_segment is None:

            compound = cq.Compound.makeCompound(
                [a.val() for a in triangle_wedges]
            )

        else:

            intersected_solids = []
            for segment in triangle_wedges:
                overlap = intersect_solid(segment, self.shape_to_segment)
                intersected_solids.append(overlap)

            compound = cq.Compound.makeCompound(
                [a.val() for a in intersected_solids]
            )

        self.solid = compound

        return compound




import warnings

import paramak


class CenterColumnStudyReactor(paramak.Reactor):
    """Creates geometry for a simple reactor that is optimised for carrying
    out parametric studies on the center column shield. Several aspects
    such as outboard magnets are intentionally missing from this reactor
    so that the model runs quickly and only includes components that have a
    significant impact on the center column shielding. This allows the
    neutronics simulations to run quickly and the column design space to be
    explored efficiently.

    Arguments:
        inner_bore_radial_thickness (float): the radial thickness of the
            inner bore (cm)
        inboard_tf_leg_radial_thickness (float): the radial thickness of
            the inner leg of the toroidal field coils (cm)
        center_column_shield_radial_thickness_mid (float): the radial thickness
            of the center column shield at the mid point (cm)
        center_column_shield_radial_thickness_upper (float): the radial
            thickness of the center column shield at the upper point (cm)
        inboard_firstwall_radial_thickness (float): the radial thickness
            of the inboard firstwall (cm)
        divertor_radial_thickness (float): the radial thickness of the divertor
            (cm)
        inner_plasma_gap_radial_thickness (float): the radial thickness of
            the inboard gap between the plasma and the center column shield
            (cm)
        plasma_radial_thickness (float): the radial thickness of the plasma
            (cm)
        outer_plasma_gap_radial_thickness (float): the radial thickness of
            the outboard gap between the plasma and the first wall (cm)
        elongation (float): the elongation of the plasma
        triangularity (float): the triangularity of the plasma
        center_column_arc_vertical_thickness (float): height of the outer
            hyperbolic profile of the center column shield.
        plasma_gap_vertical_thickness (float): the vertical thickness of
            the upper gap between the plasma and the blanket (cm)
        rotation_angle (float): the angle of the sector that is desired.
            Defaults to 360.0.
    """

    def __init__(
        self,
        inner_bore_radial_thickness,
        inboard_tf_leg_radial_thickness,
        center_column_shield_radial_thickness_mid,
        center_column_shield_radial_thickness_upper,
        inboard_firstwall_radial_thickness,
        divertor_radial_thickness,
        inner_plasma_gap_radial_thickness,
        plasma_radial_thickness,
        outer_plasma_gap_radial_thickness,
        center_column_arc_vertical_thickness,
        elongation,
        triangularity,
        plasma_gap_vertical_thickness,
        rotation_angle=360.0,
    ):

        super().__init__([])

        self.inner_bore_radial_thickness = inner_bore_radial_thickness
        self.inboard_tf_leg_radial_thickness = inboard_tf_leg_radial_thickness
        self.center_column_shield_radial_thickness_mid = \
            center_column_shield_radial_thickness_mid
        self.center_column_shield_radial_thickness_upper = \
            center_column_shield_radial_thickness_upper
        self.inboard_firstwall_radial_thickness = \
            inboard_firstwall_radial_thickness
        self.divertor_radial_thickness = divertor_radial_thickness
        self.inner_plasma_gap_radial_thickness = \
            inner_plasma_gap_radial_thickness
        self.plasma_radial_thickness = plasma_radial_thickness
        self.outer_plasma_gap_radial_thickness = \
            outer_plasma_gap_radial_thickness
        self.plasma_gap_vertical_thickness = plasma_gap_vertical_thickness
        self.center_column_arc_vertical_thickness = \
            center_column_arc_vertical_thickness
        self.rotation_angle = rotation_angle
        self.elongation = elongation
        self.triangularity = triangularity

        # adds self.input_variable_names from the Reactor class
        self.input_variable_names = self.input_variable_names + [
            'inner_bore_radial_thickness',
            'inboard_tf_leg_radial_thickness',
            'center_column_shield_radial_thickness_mid',
            'center_column_shield_radial_thickness_upper',
            'inboard_firstwall_radial_thickness',
            'divertor_radial_thickness',
            'inner_plasma_gap_radial_thickness',
            'plasma_radial_thickness',
            'outer_plasma_gap_radial_thickness',
            'center_column_arc_vertical_thickness',
            'elongation',
            'triangularity',
            'plasma_gap_vertical_thickness',
            'rotation_angle',
        ]

        # set by make_plasma
        self.major_radius = None
        self.minor_radius = None

    def create_solids(self):
        """Creates a 3d solids for each component.

           Returns:
              A list of CadQuery solids: A list of 3D solid volumes

        """
        shapes_and_components = []

        self._rotation_angle_check()
        shapes_and_components.append(self._make_plasma())
        self._make_radial_build()
        self._make_vertical_build()
        shapes_and_components.append(self._make_inboard_tf_coils())
        shapes_and_components.append(self._make_center_column_shield())
        shapes_and_components.append(self._make_inboard_firstwall())
        shapes_and_components.append(self._make_outboard_blanket())
        shapes_and_components.append(self._make_divertor())

        self.shapes_and_components = shapes_and_components

    def _rotation_angle_check(self):

        if self.rotation_angle == 360:
            msg = "360 degree rotation may result " + \
                "in a Standard_ConstructionError or AttributeError"
            warnings.warn(msg, UserWarning)

    def _make_plasma(self):

        # sets major radius and minor radius from equatorial_points to allow a
        # radial build this helps avoid the plasma overlapping the center
        # column and other components

        inner_equatorial_point = (
            self.inner_bore_radial_thickness
            + self.inboard_tf_leg_radial_thickness
            + self.center_column_shield_radial_thickness_mid
            + self.inner_plasma_gap_radial_thickness
        )
        outer_equatorial_point = \
            inner_equatorial_point + self.plasma_radial_thickness
        self.major_radius = \
            (outer_equatorial_point + inner_equatorial_point) / 2
        self.minor_radius = self.major_radius - inner_equatorial_point

        plasma = paramak.Plasma(
            major_radius=self.major_radius,
            minor_radius=self.minor_radius,
            elongation=self.elongation,
            triangularity=self.triangularity,
            rotation_angle=self.rotation_angle,
        )

        self._plasma = plasma
        return plasma

    def _make_radial_build(self):

        # this is the radial build sequence, where one component stops and
        # another starts

        self._inner_bore_start_radius = 0
        self._inner_bore_end_radius = self._inner_bore_start_radius + \
            self.inner_bore_radial_thickness

        self._inboard_tf_coils_start_radius = self._inner_bore_end_radius
        self._inboard_tf_coils_end_radius = \
            self._inboard_tf_coils_start_radius + \
            self.inboard_tf_leg_radial_thickness

        self._center_column_shield_start_radius = \
            self._inboard_tf_coils_end_radius
        self._center_column_shield_end_radius_upper = \
            self._center_column_shield_start_radius + \
            self.center_column_shield_radial_thickness_upper
        self._center_column_shield_end_radius_mid = \
            self._center_column_shield_start_radius + \
            self.center_column_shield_radial_thickness_mid

        self._inboard_firstwall_start_radius = \
            self._center_column_shield_end_radius_upper
        self._inboard_firstwall_end_radius = \
            self._inboard_firstwall_start_radius + \
            self.inboard_firstwall_radial_thickness

        self._divertor_start_radius = self._inboard_firstwall_end_radius
        self._divertor_end_radius = self._divertor_start_radius + \
            self.divertor_radial_thickness

        self._inner_plasma_gap_start_radius = \
            self._center_column_shield_end_radius_mid + \
            self.inboard_firstwall_radial_thickness

        self._inner_plasma_gap_end_radius = \
            self._inner_plasma_gap_start_radius + \
            self.inner_plasma_gap_radial_thickness

        self._plasma_start_radius = self._inner_plasma_gap_end_radius
        self._plasma_end_radius = \
            self._plasma_start_radius + \
            self.plasma_radial_thickness

        self._outer_plasma_gap_start_radius = self._plasma_end_radius
        self._outer_plasma_gap_end_radius = \
            self._outer_plasma_gap_start_radius + \
            self.outer_plasma_gap_radial_thickness

        self._outboard_blanket_start_radius = self._outer_plasma_gap_end_radius
        self._outboard_blanket_end_radius = \
            self._outboard_blanket_start_radius + 100.

    def _make_vertical_build(self):

        # this is the vertical build sequence, componets build on each other in
        # a similar manner to the radial build

        self._plasma_to_blanket_gap_start_height = self._plasma.high_point[1]
        self._plasma_to_blanket_gap_end_height = \
            self._plasma_to_blanket_gap_start_height + \
            self.plasma_gap_vertical_thickness

        self._blanket_start_height = self._plasma_to_blanket_gap_end_height
        self._blanket_end_height = self._blanket_start_height + 100.

        self._center_column_shield_end_height = self._blanket_end_height
        self._inboard_firstwall_end_height = self._blanket_end_height

    def _make_inboard_tf_coils(self):

        self._inboard_tf_coils = paramak.CenterColumnShieldCylinder(
            height=self._blanket_end_height * 2,
            inner_radius=self._inboard_tf_coils_start_radius,
            outer_radius=self._inboard_tf_coils_end_radius,
            rotation_angle=self.rotation_angle,
            name="inboard_tf_coils",
            color=(0., 0., 1.)
        )
        return self._inboard_tf_coils

    def _make_center_column_shield(self):

        self._center_column_shield = \
            paramak.CenterColumnShieldFlatTopHyperbola(
                height=self._center_column_shield_end_height * 2.,
                arc_height=self.center_column_arc_vertical_thickness,
                inner_radius=self._center_column_shield_start_radius,
                mid_radius=self._center_column_shield_end_radius_mid,
                outer_radius=self._center_column_shield_end_radius_upper,
                rotation_angle=self.rotation_angle)
        return self._center_column_shield

    def _make_inboard_firstwall(self):

        self._inboard_firstwall = paramak.InboardFirstwallFCCS(
            central_column_shield=self._center_column_shield,
            thickness=self.inboard_firstwall_radial_thickness,
            rotation_angle=self.rotation_angle)
        return self._inboard_firstwall

    def _make_outboard_blanket(self):

        self._center_column_cutter = paramak.CenterColumnShieldCylinder(
            # extra 1.5 to ensure overlap,
            height=self._inboard_firstwall_end_height * 2.5,
            inner_radius=0,
            outer_radius=self._inboard_firstwall_end_radius,
            rotation_angle=self.rotation_angle
        )

        self._blanket = paramak.BlanketFP(
            plasma=self._plasma,
            thickness=100.,
            offset_from_plasma=[
                self.inner_plasma_gap_radial_thickness,
                self.plasma_gap_vertical_thickness,
                self.outer_plasma_gap_radial_thickness,
                self.plasma_gap_vertical_thickness,
                self.inner_plasma_gap_radial_thickness],
            start_angle=-180,
            stop_angle=180,
            color=(0., 1., 0.498),
            rotation_angle=self.rotation_angle,
            cut=[self._center_column_cutter]
        )
        return self._blanket

    def _make_divertor(self):
        self._blanket_enveloppe = paramak.BlanketFP(
            plasma=self._plasma,
            thickness=100.,
            offset_from_plasma=[
                self.inner_plasma_gap_radial_thickness,
                self.plasma_gap_vertical_thickness,
                self.outer_plasma_gap_radial_thickness,
                self.plasma_gap_vertical_thickness,
                self.inner_plasma_gap_radial_thickness],
            start_angle=-180,
            stop_angle=180,
            rotation_angle=self.rotation_angle,
            cut=[self._center_column_cutter]
        )

        self._divertor = paramak.CenterColumnShieldCylinder(
            height=self._center_column_shield_end_height *
            2.5,  # extra 0.5 to ensure overlap
            inner_radius=self._divertor_start_radius,
            outer_radius=self._divertor_end_radius,
            rotation_angle=self.rotation_angle,
            name="divertor",
            intersect=self._blanket_enveloppe,
            color=(1., 0.667, 0.),
        )
        self._blanket.cut.append(self._divertor)
        return self._divertor




import cadquery as cq
from paramak import CuttingWedgeFS, Shape


class ShellFS(Shape):
    """Shell From Shape. Creates a shell casing for the provided shape object.
    Warning some shapes are too complex to shell. If using a rotated shape then
    setting the rotation angle to 360 can simplify the shell. Then intersecting
    the resulting shell with the WedgeCutterFs can reduce the shell back down
    to the original rotation angle of the shape.

    Args:
        shape (paramak.Shape): the shape to create a shell / 3D offset around
        thickness (float): the thickness of the shell casing around the shape
            (cm). Passed directly to CadQuery.shell(). Defaults to 10.0.
        kind (str, optional) : the method used to connect gaps in the resulting
            shelled shape. Options include 'arc' or 'intersection'. Use 'arc'
            for rounded edges 'intersection' for sharp edges. Passed directly
            to CadQuery.shell(). Defaults to intersection.
    """

    def __init__(
        self,
        shape,
        thickness=10.0,
        kind='intersection',
        **kwargs
    ):

        super().__init__(
            **kwargs
        )

        self.shape = shape
        self.thickness = thickness
        self.kind = kind

    @property
    def shape(self):
        return self._shape

    @shape.setter
    def shape(self, value):
        self._shape = value

    @property
    def thickness(self):
        return self._thickness

    @thickness.setter
    def thickness(self, thickness):
        self._thickness = thickness

    def create_solid(self):
        """Creates a 3D solid by creating a shell around a Shape"""

        if isinstance(self.shape, cq.Workplane):
            solid = self.shape.shell(
                thickness=self.thickness,
                kind=self.kind,
            )
        else:
            solid = self.shape.solid.shell(
                thickness=self.thickness,
                kind=self.kind,
            )

        self.solid = solid

        return solid




import cadquery as cq
from paramak import Shape


class HollowCube(Shape):
    """A hollow cube with a constant thickness. Can be used to create a DAGMC
    Graveyard.

    Arguments:
        length (float): The length to use for the height, width, depth of the
            inner dimensions of the cube.
        thickness (float, optional): thickness of the vessel. Defaults to 10.0.
    """

    def __init__(
        self,
        length,
        thickness=10.,
        **kwargs
    ):
        self.length = length
        self.thickness = thickness
        super().__init__(
            **kwargs
        )

    @property
    def thickness(self):
        return self._thickness

    @thickness.setter
    def thickness(self, value):
        self._thickness = value

    @property
    def length(self):
        return self._length

    @length.setter
    def length(self, value):
        self._length = value

    def create_solid(self):

        # creates a small box that surrounds the geometry
        inner_box = cq.Workplane("front").box(
            self.length,
            self.length,
            self.length
        )

        # creates a large box that surrounds the smaller box
        outer_box = cq.Workplane("front").box(
            self.length + self.thickness,
            self.length + self.thickness,
            self.length + self.thickness
        )

        # subtracts the two boxes to leave a hollow box
        new_shape = outer_box.cut(inner_box)

        self.solid = new_shape

        return new_shape




from typing import Optional
import numpy as np
from paramak import ExtrudeCircleShape


class CoolantChannelRingStraight(ExtrudeCircleShape):
    """A ring of equally-spaced straight circular coolant channels with
    constant thickness.

    Args:
        height: height of each coolant channel in ring.
        channel_radius: radius of each coolant channel in ring.
        number_of_coolant_channels: number of coolant channels in ring.
        ring radius: radius of coolant channel ring.
        start_angle: angle at which the first channel in the ring is placed.
            Defaults to 0.0.
        rotation_axis (str, optional): azimuthal axis around which the separate
            coolant channels are placed.
        workplane (str, optional): plane in which the cross-sections of the
            coolant channels lie. Defaults to "XY".
    """

    def __init__(
        self,
        height: float,
        channel_radius: float,
        number_of_coolant_channels: int,
        ring_radius: float,
        start_angle: Optional[float] = 0.0,
        **kwargs
    ) -> None:

        super().__init__(
            distance=height,
            radius=channel_radius,
            **kwargs
        )

        self.height = height
        self.channel_radius = channel_radius
        self.number_of_coolant_channels = number_of_coolant_channels
        self.ring_radius = ring_radius
        self.start_angle = start_angle

    @property
    def azimuth_placement_angle(self):
        self.find_azimuth_placement_angle()
        return self._azimuth_placement_angle

    @azimuth_placement_angle.setter
    def azimuth_placement_angle(self, value):
        self._azimuth_placement_angle = value

    def find_azimuth_placement_angle(self):
        """Calculates the azimuth placement angles based on the number of
        coolant channels."""

        angles = list(
            np.linspace(
                0 + self.start_angle,
                360 + self.start_angle,
                self.number_of_coolant_channels,
                endpoint=False))

        self.azimuth_placement_angle = angles

    def find_points(self):

        points = [(self.ring_radius, 0)]

        self.points = points




from paramak import ITERtypeDivertor


class ITERtypeDivertorNoDome(ITERtypeDivertor):
    """Creates an ITER-like divertor with inner and outer vertical targets
    """

    def __init__(
        self,
        **kwargs
    ):

        super().__init__(
            dome=False,
            dome_height=None,
            dome_length=None,
            dome_thickness=None,
            dome_pos=None,
            **kwargs
        )




from typing import Optional, List, Tuple

from cadquery import Workplane
from paramak import Shape


class SweepCircleShape(Shape):
    """Sweeps a 2D circle of a defined radius along a defined spline path to
    create a 3D CadQuery solid. Note, some variation in the cross-section of
    the solid may occur.

    Args:
        radius: Radius of 2D circle to be swept.
        path_points: A list of XY, YZ or XZ coordinates connected by spline
            connections which define the path along which the 2D shape is swept
        workplane: Workplane in which the circle to be swept
            is defined. Defaults to "XY".
        path_workplane: Workplane in which the spline path is
            defined. Defaults to "XZ".
        force_cross_section: If True, cross-section of solid is forced to be
            shape defined by points in workplane at each path_point. Defaults
            to False.
    """

    def __init__(
        self,
        radius: float,
        path_points: List[Tuple[float, float]],
        workplane: Optional[str] = "XY",
        path_workplane: Optional[str] = "XZ",
        force_cross_section: Optional[bool] = False,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.651, 0.808, 0.89),
        **kwargs
    ):

        super().__init__(
            workplane=workplane,
            color=color,
            **kwargs
        )

        self.radius = radius
        self.path_points = path_points
        self.path_workplane = path_workplane
        self.force_cross_section = force_cross_section

    @property
    def radius(self):
        return self._radius

    @radius.setter
    def radius(self, value):
        self._radius = value

    @property
    def path_points(self):
        return self._path_points

    @path_points.setter
    def path_points(self, value):
        self._points = value
        self._path_points = value

    @property
    def path_workplane(self):
        return self._path_workplane

    @path_workplane.setter
    def path_workplane(self, value):
        if value[0] != self.workplane[0]:
            raise ValueError(
                "workplane and path_workplane must start with the same letter"
            )
        elif value == self.workplane:
            raise ValueError(
                "workplane and path_workplane must be different"
            )
        else:
            self._path_workplane = value

    def create_solid(self):
        """Creates a swept 3D solid from a 2D circle.

        Returns:
            A CadQuery solid: A 3D solid volume
        """

        path = Workplane(self.path_workplane).spline(self.path_points)

        factor = 1
        if self.workplane in ["XZ", "YX", "ZY"]:
            factor *= -1

        wires = []
        if self.force_cross_section:
            wire = Workplane(self.workplane).center(0, 0)
            for point in self.path_points[:-1]:
                wire = (
                    wire.workplane(offset=point[1] * factor)
                    .center(point[0], 0)
                    .circle(self.radius)
                )

                wires.append(wire)

                wire = (
                    wire.center(-point[0], 0)
                    .workplane(offset=-point[1] * factor)
                )

            self.wire = wires

            solid = wire.workplane(
                offset=self.path_points[-1][1] * factor) \
                .center(self.path_points[-1][0], 0) \
                .circle(self.radius) \
                .sweep(path, multisection=True)

        else:

            wire = (
                Workplane(self.workplane)
                .workplane(offset=self.path_points[0][1] * factor)
                .center(self.path_points[0][0], 0)
                .workplane()
                .circle(self.radius)
                .center(-self.path_points[0][0], 0)
                .workplane(offset=-self.path_points[0][1] * factor)
                .workplane(offset=self.path_points[-1][1] * factor)
                .center(self.path_points[-1][0], 0)
                .workplane()
                .circle(self.radius)
            )

            self.wire = wire

            solid = wire.sweep(path, multisection=True)

        solid = self.rotate_solid(solid)
        solid = self.perform_boolean_operations(solid)
        self.solid = solid
        return solid



import pytest
import math
import paramak

'''This test evaluates the perimeter of the resultant points and asserts them agaist the known value'''


def perimeter(outer_start_point, radius, thickness):

    test_shape = paramak.CapsuleVacuumVessel(
        outer_start_point=outer_start_point,
        radius=radius,
        thickness=thickness
    )
    point1 = test_shape.points[0]
    point2 = test_shape.points[1]
    point3 = test_shape.points[2]
    point4 = test_shape.points[3]
    point5 = test_shape.points[4]
    point6 = test_shape.points[5]
    point7 = test_shape.points[6]
    point8 = test_shape.points[7]
    point9 = test_shape.points[8]
    point10 = test_shape.points[9]
    point11 = test_shape.points[10]
    point12 = test_shape.points[11]

    straightedges = float((point12[1] -
                           point1[1]) +
                          (point6[1] -
                           point7[1]) +
                          (point4[1] -
                           point3[1]) +
                          (point9[1] -
                           point10[1]))
    curvededges = float((math.pi * radius) + (math.pi * (radius - thickness)))
    total = float(straightedges + curvededges)

    return total


def test_perimeter1():
    testp = perimeter(
        outer_start_point=(0, 0),
        radius=300,
        thickness=10)
    assert testp == pytest.approx(3073.54)


def test_perimeter2():
    testp = perimeter(
        outer_start_point=(100, -100),
        radius=400,
        thickness=25)
    assert testp == pytest.approx(4084.734)


def test_perimeter3():
    testp = perimeter(
        outer_start_point=(1000, -500),
        radius=5000,
        thickness=50)
    assert testp == pytest.approx(51358.85)


def test_pointnum1():
    """this tests if the number of points returned are correct"""
    shape = paramak.CapsuleVacuumVessel(
        outer_start_point=(0, 0),
        radius=300,
        thickness=10)
    assert len(shape.points) == 12
    assert len(shape.processed_points) == 13


def test_pointnum2():
    """this tests if the number of points returned are correct"""
    shape = paramak.CapsuleVacuumVessel(
        outer_start_point=(100, -100),
        radius=400,
        thickness=25)
    assert len(shape.points) == 12
    assert len(shape.processed_points) == 13


def test_pointnum3():
    """this tests if the number of points returned are correct"""
    shape = paramak.CapsuleVacuumVessel(
        outer_start_point=(1000, -500),
        radius=5000,
        thickness=50)
    assert len(shape.points) == 12
    assert len(shape.processed_points) == 13


'''this tests the volume of the created component and asserts it agaist the known value'''


def volume(outer_start_point, radius, thickness, angle):

    test_shape = paramak.CapsuleVacuumVessel(
        outer_start_point=outer_start_point,
        radius=radius,
        thickness=thickness,
        rotation_angle=angle
    )

    point3 = test_shape.points[2]
    point4 = test_shape.points[3]

    point9 = test_shape.points[8]
    point10 = test_shape.points[9]

    outer_volume_cylinder = float(
        math.pi * (radius**2) * (point4[1] - point3[1]))
    inner_volume_cylinder = float(
        math.pi * ((radius - thickness)**2) * (point9[1] - point10[1]))
    outer_volume_sphere = float((4 / 3) * math.pi * (radius**3))
    inner_volume_sphere = float((4 / 3) * math.pi * ((radius - thickness)**3))
    outer_volume = outer_volume_sphere + outer_volume_cylinder
    inner_volume = inner_volume_cylinder + inner_volume_sphere
    total_volume = (angle / 360) * (outer_volume - inner_volume)

    return total_volume


def testvolume1():
    testvol = volume(
        outer_start_point=(0, 0),
        radius=300,
        thickness=10,
        angle=180)
    assert testvol == pytest.approx(11029084.61)


def testvolume2():
    testvol = volume(
        outer_start_point=(100, -100),
        radius=400,
        thickness=25,
        angle=360)
    assert testvol == pytest.approx(95884025.78)


def testvolume3():
    testvol = volume(
        outer_start_point=(1000, -500),
        radius=5000,
        thickness=50,
        angle=90)
    assert testvol == pytest.approx(7795207671.41)




import math
import random
import unittest

import paramak
import pytest


class TestCuttingWedge(unittest.TestCase):
    """Creates a random sized cutting wedge and changes the volume"""

    def test_volume_of_for_5_random_dimentions(self):
        for test_number in range(5):
            height = random.uniform(1., 2000.)
            radius = random.uniform(1., 1000)
            rotation_angle = random.uniform(1., 360.)
            azimuth_placement_angle = random.uniform(1., 360.)

            test_shape = paramak.CuttingWedge(
                height=height,
                radius=radius,
                rotation_angle=rotation_angle,
                azimuth_placement_angle=azimuth_placement_angle
            )
            angle_fraction = 360 / rotation_angle
            correct_volume = (math.pi * radius ** 2 * height) / angle_fraction
            assert test_shape.volume() == pytest.approx(correct_volume)
